jQuery(document).ready(function($) {
	
	cfswp_initGridSearchform($);	
	cfswp_initTextSearch($);	
	cfswp_initThings($);	
	cfswp_initCarouselNav($);	
	cfswp_initContactFlag($);	
	
});


	
/***********************************
*  grid search
***********************************/
function cfswp_initGridSearchform($) {
	
	// submit searchform
	if ($('.cfs-searchbar').length > 0 ) {
		$('.cfs-searchbar #searchtext').change(function(){
			if ($(this).val() == '') {
				$(this).parent().removeClass('active');
			}
			else {
				$(this).parent().addClass('active');
			}
			var id = $(this).closest('.cfs-searchbar').attr('id');
			cfswp_submitGridSearchForm($, id);
		});
		$('.cfs-searchbar .searchicon').click(function(){
			var id = $(this).closest('.cfs-searchbar').attr('id');
			cfswp_submitGridSearchForm($, id);
		});
		$('.cfs-searchbar .reset').click(function(){
			var id = $(this).closest('.cfs-searchbar').attr('id');
			$(this).parent().removeClass('active');
			cfswp_resetGridSearch(id, 'text');
		});
		$('.cfs-searchbar .searchlinks a, .cfs-searchbar .sortlinks a').click(function(){
			if ($(this).hasClass('active')) {
				$(this).removeClass('active');
			}
			else {
				$(this).siblings('a').removeClass('active');
				$(this).addClass('active');
			}
			
			var id = $(this).closest('.cfs-searchbar').attr('id');
			cfswp_submitGridSearchForm($, id);
		});
		$('.cfs-searchbar .modelinks a').click(function(){
			if (!$(this).hasClass('active')) {
				$(this).siblings('a').removeClass('active');
				$(this).addClass('active');
				
				var id = $(this).closest('.cfs-searchbar').attr('id');
				cfswp_submitGridSearchForm($, id);
			}			
		});
	}
	
	// scroll to searchform
	if ($('.cfs-searchbar').length > 0) {
		
		const urlSearchParams = new URLSearchParams(window.location.search);
		const getparams = Object.fromEntries(urlSearchParams.entries());
				
		if (getparams.hasOwnProperty('searchtext') || getparams.hasOwnProperty('searchcat') || getparams.hasOwnProperty('searchtag')) {
			$('html, body').animate({
				scrollTop: $(".cfs-searchbar").offset().top - $('#page-header').outerHeight() - cfswp_scrollTopOffset
			}, cfswp_animationSpeed);
		}
	}
	
}

function cfswp_submitGridSearchForm($, id) {
	
	var target = $('#' + id).attr('data-target').split(',');
	
	// get search params
	var text = $('#' + id + ' input#searchtext').length > 0 ? $('#' + id + ' input#searchtext').val() : '';
	var cat = $('#' + id + ' .searchlinks a.active[data-target="cat"]').length > 0 ? $('#' + id + ' .searchlinks a.active[data-target="cat"]').attr('data-value') : '';
	var tag = $('#' + id + ' .searchlinks a.active[data-target="tag"]').length > 0 ? $('#' + id + ' .searchlinks a.active[data-target="tag"]').attr('data-value'): '';
	var sort = $('#' + id + ' .sortlinks a.active[data-target="sort"]').length > 0 ? $('#' + id + ' .sortlinks a.active[data-target="sort"]').attr('data-value'): '';
	var mode = $('#' + id + ' .modelinks a.active[data-target="mode"]').length > 0 ? $('#' + id + ' .modelinks a.active[data-target="mode"]').attr('data-value'): '';
	
	var data = {};
	data.action = 'cfsGridSearch';	
	data.nonce = cfsgrid_ajax_object.nonce;
	
	if (text != '') data.text = text;
	if (cat != '') data.cat = cat;
	if (tag != '') data.tag = tag;
	if (sort != '') data.sort = sort;	
	if (mode != '') data.mode = mode;	
		
	// get grid configuration
	var config = $('#' + id + ' input[data-target="config"]');
	$.each(config, function(index, item) {
		var name = $(item).attr('name');
		data['config_' + name] = $(item).val();
	});
	
	// refresh target list
	jQuery.each(target, function(index, tgt) {
		data.target = tgt;
		jQuery.ajax({
			type: 'POST',
			url: cfsgrid_ajax_object.ajaxurl,
			dataType: "html",
			data: data,
			success: function (result, textStatus, XMLHttpRequest) {
				if ($('#'+tgt).is(":visible")) {
					$('#'+tgt).fadeOut(100, function() {
						$('#'+tgt).html(result).fadeIn(100, function() {
							cfswp_initThings($, '#'+tgt);
						});
					});
				}
				else {
					$('#'+tgt).html(result);
					cfswp_initThings($, '#'+tgt);
				}
			},
			error: function (XMLHttpRequest, textStatus, errorThrown) {
				console.log(errorThrown);	
			}
		});
	});
	
}

function cfswp_resetSearchLabel(id, tgt) {
	cfswp_resetGridSearch(id + '-searchbar', tgt);
}
function cfswp_resetGridSearch(id, tgt) {	
	if (tgt == 'text') {
		jQuery('#' + id + ' input#searchtext').val('');
	}
	else {
		jQuery('#' + id + ' .searchlinks a.active[data-target="'+tgt+'"]').removeClass('active');
	}
	cfswp_submitGridSearchForm(jQuery, id)
}
function cfswp_setGridSearchParams(params) {	
	
	var id = jQuery('.cfs-searchbar').attr('id');
	if (params.hasOwnProperty('searchtext') && params.searchtext !== null) {
		jQuery('#' + id + ' input#searchtext').val(params.searchtext);
	}
	if (params.hasOwnProperty('searchcat') && params.searchcat !== null) {
		jQuery('#' + id + ' .searchlinks a[data-target="cat"][data-value="'+params.searchcat+'"]').addClass('active');
	}
	if (params.hasOwnProperty('searchtag') && params.searchtag !== null) {
		jQuery('#' + id + ' .searchlinks a[data-target="tag"][data-value="'+params.searchtag+'"]').addClass('active');
	}
}

/***************************************************
* lightbox, carousel, activatable, toggle
***************************************************/
function cfswp_initThings($, container) {
	
	if (typeof(container) === 'undefined') {
		container = '';
	} else {
		container = container + ' ';
	}
	
	// lightbox images + gallery
	var lightboxSelector = container + '.cfs-lightbox';
	var lightboxGallery = container + '.cfs-lightbox-gallery';
	if ($(lightboxSelector + ', ' + lightboxGallery).length > 0) {	
		$(lightboxSelector + ', ' + lightboxGallery).each(function(index, element) {	
			var lightboxOptions = cfswp_getLightboxOptions($, element);
			$(element).magnificPopup(lightboxOptions);
		});
	}
	
	// carousel
	var carouselSelector = container + '.cfs-carousel';
	if ($(carouselSelector).length > 0) {
		$(carouselSelector).each(function(index, element) {	
			var carouselOptions = cfswp_getCarouselOptions($, element);
			$(element).owlCarousel(carouselOptions);
		});
	}
	
	// activate 
	var activatableSelector = container + '.cfs-activate';	
	if ($(activatableSelector).length > 0) {
		$(activatableSelector).click(function(event) {
			cfswp_clickActivatable($, this, event);
		});
		$(window).click(function(event) {       
			if ($(".mfp-ready").length == 0) {
				$('.cfs-activate.activated').removeClass('activated');  		
			}
		});
	}

	// toggle display
	var toggleSelector = container + '.cfs-toggle';	
	if ($(toggleSelector).length > 0) {
		$(toggleSelector).each(function(index, element) {
			$(element).click(function(event) { console.log(this);
				cfswp_clickToggle($, this, event);
			});
		});
	}

}

function cfswp_initCarouselNav($) {
	
	var hash = $(location).attr('hash').substr(1);
	
	$('.cfs-carousel-searchbar').each(function(i, searchbar) {
		var gridId = $(searchbar).attr('data-target');
		if ($(searchbar).find('.navilinks a').length > 0) {
			
			// set slide by menu
			$(searchbar).find('.navilinks a').each(function(index, element) {
				$(element).click(function(){
					$('#'+gridId).find('.cfs-carousel').trigger('to.owl.carousel', index);
					$(searchbar).find('.navilinks a.active').removeClass('active');
					$(this).addClass('active');
				});
			});	

			// set menu by slide
			$('#'+gridId).find('.cfs-carousel').on('changed.owl.carousel', function(event) {
				var k = event.item.index;
				$(searchbar).find('.navilinks a.active').removeClass('active');
				$(searchbar).find('.navilinks a:nth-child(' + (k + 1) + ')').addClass('active');
			});		
			
			// set slide by url hash
			if (hash.length > 0 && $(searchbar).find('.navilinks a[data-value=' + hash + ']').length > 0) {
				$(searchbar).find('.navilinks a.active').removeClass('active');
				$(searchbar).find('.navilinks a[data-value=' + hash + ']').addClass('active').trigger('click');
				$([document.documentElement, document.body]).animate({
					scrollTop: $(searchbar).offset().top - $('#page-header').outerHeight() - cfswp_scrollTopOffset
				}, cfswp_animationSpeed);
			}
		}
	});
}
	
function cfswp_clickToggle($, element, event) {
	
	var tgt = $(element).attr('data-target');
	if (typeof(tgt) !== 'undefined' && $(tgt).length > 0) {
		if ($(tgt).is(':visible')) {
			$(element).removeClass('active');
			$(tgt).slideUp(function(){
				$(tgt).removeClass('active');
			});
		}
		else {
			$(element).addClass('active'); 
			$(tgt).slideDown(function(){
				$(tgt).addClass('active');
			});					
		}
	}
}

function cfswp_clickActivatable($, element, event) {
	
	var target = $(element).attr('data-target');
	if (typeof target === 'undefined' || target === false) {
		target = element;
		event.stopPropagation();
	}
	if (!$(target).hasClass('activated')) {
		$(target).siblings('.activated').removeClass('activated');
		$(target).addClass('activated');

		// scroll top
		var scrollto = $(target);
		var isMobile = $('body').hasClass('state_tablets') || $('body').hasClass('state_mobiles');	
		if (isMobile) {
			setTimeout(function(){
				$([document.documentElement, document.body]).animate({
					scrollTop: scrollto.offset().top - $('#page-header').outerHeight() - cfswp_scrollTopOffset
				}, cfswp_animationSpeed);
			}, cfswp_animationSpeed);
		}
	}
}

/***********************************
*  text search by HTML content
***********************************/
function cfswp_initTextSearch($) {
	if ($('input.cfs-textsearch').length > 0) {
		$("input.cfs-textsearch").on('keyup', function (event) {
			cfswp_performTextSearch($, this, event);
		});
	}
}


function cfswp_performTextSearch($, element, event) {
	
	var tgt = $(element).attr('data-target');
	if (typeof tgt === 'undefined' || tgt == '') {
		console.log('cfs-textsearch data-target missing.');
		return;
	}

	var searchtext = $(element).val();
	if (searchtext != '') {
		$(tgt).each(function(index, item) {				
			var content = $(item).text();
			var found = content.toUpperCase().indexOf(searchtext.toUpperCase());
			if (found > -1) {
				$(item).show();
			}
			else {
				$(item).hide();
			}
		});
	}
	else {
		$(tgt).show();
	}

	var noresults = $(element).closest('.cfs-textsearch-container').find('.cfs-textsearch-noresults');
	if (noresults.length > 0) {
		if ($(tgt+':visible').length == 0) {
			noresults.show();
		}
		else {
			noresults.hide();
		}
	}
}

/***********************************
*  contact flag
***********************************/
function cfswp_initContactFlag($) {	
	if ($('#contact-flag a.toggler').length > 0) {
		$('#contact-flag a.toggler').click(function(){
			$(this).closest('#contact-flag').toggleClass('active');
		});
	}
}


/**********************************
 * PARALLAX scroll effect
 **********************************/ 
var parallax_start = '';
function initParallax($, target, speed, dir, mode, head) {
		
	if (typeof(target) === 'undefined' || target === false) {
		target = '.parallax-scroll';
	}
	if (typeof(speed) === 'undefined' || speed === false) {
		speed = 5;
	}
	if (typeof(dir) === 'undefined' || dir === false || dir != 'x') {
		dir = 'y';
	}
	if (typeof(mode) === 'undefined' || mode === false) {
		mode = 'translate';
	}
	if (typeof(head) === 'undefined') {
		head = false;
	}
	
	$(window).on('DOMContentLoaded load resize scroll', function(){
		
		// screen position
		var top_of_screen = $(window).scrollTop();
		var bottom_of_screen = $(window).scrollTop() + $(window).innerHeight();
		
		$(target).each(function(){ 
			if (isElementInView($, $(this), false)) {
				
				// element position
				var top_of_element = $(this).offset().top;
				var bottom_of_element = $(this).offset().top + $(this).outerHeight();
							
				// scrolled distance
				if (head) {
					var dist = top_of_screen; 
				}
				else {
					var dist = bottom_of_screen - top_of_element; 
				}
				if (dist < 0) dist = 0;
				
				// translation
				var d = parseInt(-1/100 * speed * dist);
				
				if (mode == 'top' || mode == 'bottom' || mode == 'left' || mode == 'right') {
					if (parallax_start === '') parallax_start = parseInt($(this).css(mode));
					var pos = parallax_start + d;
					$(this).css(mode, pos + 'px');
				}
				else if (mode == 'bg') {
					var bgmode = 'background-position-' + dir;
					if (parallax_start === '') parallax_start = parseInt($(this).css(bgmode));
					var pos = parallax_start + d;
					$(this).css(bgmode, pos + 'px');
				}
				else {
					$(this).css('transform', 'translate' + (dir=='y' ? 'Y' : 'X') +'(' + d + 'px)');	
				}
			}
		});
	});
}

function isElementInView ($, element, fullyInView) {
	var pageTop = $(window).scrollTop();
	var pageBottom = pageTop + $(window).height();
	var elementTop = $(element).offset().top;
	var elementBottom = elementTop + $(element).outerHeight();

	if (fullyInView === true) {
		return ((pageTop < elementTop) && (pageBottom > elementBottom));
	} else {
		return ((elementTop <= pageBottom) && (elementBottom >= pageTop));
	}
}
function setElementInView($, element) {
	if (isElementInView(element, true)) {
		$(element).addClass('in-view');
	}
	if (!isElementInView(element, false)) {
		$(element).removeClass('in-view');
	}
}	