<?php
/* Do not access this file directly */
defined('ABSPATH') or die('');

if ( ! class_exists( 'CFSWP_Features' ) ) {
	
	class CFSWP_Features {
		
		var $options = array();
		var $GET = array();
		var $POST = array();
				
		public function __construct() {
			
			/* admin scripts and styles */
			add_action('admin_enqueue_scripts', function( $hook ) {				
				if( $hook == 'settings_page_cfswp-settings' ) {
					wp_enqueue_style( 'cfswp-admin-css', CFSWP_PLUGIN_URL . 'assets/css/cfswp-admin.css' );
					wp_enqueue_script( 'cfswp-admin-js', CFSWP_PLUGIN_URL . 'assets/js/cfswp-admin.js' );
				}
			});
				
			/* public scripts and styles + ajax object */
			add_action('wp_enqueue_scripts', function() {
				
				// 3rd party scripts
				wp_enqueue_style( 'lightbox-css', CFSWP_PLUGIN_URL . 'assets/css/magnific-popup.css' );
				wp_enqueue_script( 'lightbox-js', CFSWP_PLUGIN_URL . 'assets/js/magnific-popup.js', array('jquery') );
				wp_enqueue_script( 'slider-js', CFSWP_PLUGIN_URL . 'assets/js/owl.carousel.js', array('jquery') );
				wp_enqueue_script( 'slider-thumbs-js', CFSWP_PLUGIN_URL . 'assets/js/owl.carousel2.thumbs.js', array('jquery') );

				// plugin scripts
				wp_enqueue_style( 'cfswp-css', CFSWP_PLUGIN_URL . 'assets/css/cfswp.css');
				wp_enqueue_script( 'cfswp-options-js', CFSWP_PLUGIN_URL . 'assets/js/cfswp-options.js', array('jquery') );	
				wp_enqueue_script( 'cfswp-js', CFSWP_PLUGIN_URL . 'assets/js/cfswp.js', array('jquery') );	
				
				// ajax object
				wp_localize_script( 'cfswp-js', 'cfsgrid_ajax_object', array(
					'ajaxurl' => admin_url( 'admin-ajax.php' ),
                	'nonce' => wp_create_nonce( 'cfsgrid_search_nonce' ),
				));
			});

			/* ajax actions */
			add_action( 'wp_ajax_nopriv_cfsGridSearch', array($this, 'ajax_gridsearch') );
			add_action( 'wp_ajax_cfsGridSearch', array($this, 'ajax_gridsearch') );
			
			/* feature actions */
			add_action( 'template_redirect', array($this, 'redirect_standard_pages') );
			if (class_exists('ACF')) {
				add_filter( 'acf/save_post', array($this, 'save_gallery_to_thumbnail') );
			}
			
			/* shortcodes */			
			add_shortcode('cfs-grid', array($this, 'render_grid') ); 
			add_shortcode('cfs-textsearch', array($this, 'render_textsearch') ); 
			add_shortcode('cfs-template', array($this, 'get_custom_template')); 
			if (class_exists('ACF')) {
				add_shortcode('cfs-gallery', array($this, 'render_gallery') ); 
			}
			
			/* vc elements */		
			if (is_admin()) {
				add_action( 'vc_before_init', array($this, 'vc_map_element_cfsgrid') );
				add_action( 'vc_before_init', array($this, 'vc_map_element_textsearch') );
			}
			
			/* get options */
			$this->options = get_option( 'cfswp_options' );
			
			/* collect input vars */
			$this->collectInputVars();
			
			/* set optional config defines */
			$this->setConfigDefines();
		}

		
		/****************************************
		 * get a set of options
		 ****************************************/
		private function getOptions($keys) {
			$options = array();
			foreach ($keys as $k) {
				$options[$k] = !empty($this->options[$k]) ? $this->options[$k] : false;
			}
			return $options;
		}
		
		/****************************************
		 * sanitize input vars
		 ****************************************/
		private function collectInputVars() {
			$this->GET = $this->loopInputVars($_GET);
			$this->POST = $this->loopInputVars($_POST);
		}
		
		private function loopInputVars($vars) {
			$res = array();
			foreach ($vars as $k => $v) {
				$key = $this->sanitize($k);
				if (is_array($v)) {
					$res[$key] = $this->loopInputVars($v);
				}
				else {
					$res[$key] = $this->sanitize($v);
				}
			}
			return $res;
		}
		
		private function sanitize($value) {
			return htmlspecialchars(strip_tags($value));
		}
		
		/****************************************
		 * config defines
		 ****************************************/
		private function setConfigDefines() {
			$options = $this->getOptions(array(
				'cfswp_allow_unfiltered_uploads',
				'cfswp_allow_unfiltered_uploads_date',
				'cfswp_debug_mode',
			));			
			
			// check date for unfiltered uploads
			if (!empty($options['cfswp_allow_unfiltered_uploads'])) {
				$expired = empty($options['cfswp_allow_unfiltered_uploads_date']) || strtotime($options['cfswp_allow_unfiltered_uploads_date']) < strtotime('yesterday');
				
				if ($expired) {
					$this->options['cfswp_allow_unfiltered_uploads'] = $options['cfswp_allow_unfiltered_uploads'] = '';
					$this->options['cfswp_allow_unfiltered_uploads_date'] = $options['cfswp_allow_unfiltered_uploads_date'] = '';
					update_option('cfswp_options', $this->options);
				}

				if (is_admin() && !empty($options['cfswp_allow_unfiltered_uploads'])) {
					define('ALLOW_UNFILTERED_UPLOADS', true);
				}
			}
			
		}
		
		/****************************************
		 * redirect archive and single
		 ****************************************/
		public function redirect_standard_pages() {
			
			$options = $this->getOptions(array(
				'cfswp_redirect_archives',
				'cfswp_redirect_archives_except',
				'cfswp_redirect_singles',
				'cfswp_redirect_singles_except',
			));
				
			do_action('cfswp_redirect_rules', $options);

			// redirect archives
			if (!empty($options['cfswp_redirect_archives'])) {
				if( is_archive() ) {			
					// check archive page against category exception	
					$except = !empty($options['cfswp_redirect_archives_except']) ? explode(' ', $options['cfswp_redirect_archives_except']) : array();
					$show = !empty($except) && has_category($except);			
					if( !$show ) {
						wp_redirect( home_url(), 301 );
						exit();
					}
				}
			}	
			// redirect singles
			if (!empty($options['cfswp_redirect_singles'])) {
				if( is_single() ) {		
					// check single page against posttype and category exception	
					$except = !empty($options['cfswp_redirect_singles_except']) ? explode(' ', $options['cfswp_redirect_singles_except']) : array();
					$show = !empty($except) && ( is_singular($except) || has_category($except) );			
					if( !$show ) {
						wp_redirect( home_url(), 301 );
						exit();
					}
				}
			}
		}

		/*******************************
		 * custom template
		 *******************************/
		function get_custom_template($attr) {

			$args = shortcode_atts( array(    
				'type' => 'post',  
				'args' => ''
			), $attr );
			
			$no_whitespaces = preg_replace( '/\s*,\s*/', ',', filter_var( $args['args'], FILTER_SANITIZE_STRING ) );
			$tplargs = explode(',', $no_whitespaces);
			
			ob_start();
			get_template_part( 'templates/custom', $args['type'], $tplargs );
			return ob_get_clean();
		}
		
		/************************************************************
		 * textsearch input field
		 ************************************************************/
		public function render_textsearch($attr) {

			$args = shortcode_atts( array(    
				'target' => '',  
				'id' => '',
				'class' => '',
				'placeholder' => '',
				'icon' => '',
				'noresults' => '',
			), $attr );


			$result = '<div class="cfs-textsearch-container' 
				. (!empty($args['class']) ? ' ' . $args['class'] : '') . '"' 
				. (!empty($args['id']) ? ' id="' . $args['id'] . '"' : '') . '>';

			$result .= '  <div class="cfs-textsearch-input">';
			$result .= '		<input class="cfs-textsearch" type="text" value="" data-target="'.$args['target'].'" placeholder="'.$args['placeholder'].'" />';
			if (!empty($args['icon'])) {
				$result .= '<i class="searchicon fas fa-search"></i>';
			}
			$result .= '	</div>';

			if (!empty($args['noresults'])) {
				$result .= '  <div class="cfs-textsearch-noresults" style="display:none">';
				$result .= '<p>'.$args['noresults'].'</p>';
				$result .= '	</div>';
			}
			$result .= '</div>';

			return apply_filters('cfswp_render_textsearch', $result, $args);
		}
	

		/************************************************************
		 * grid search ajax wrapper
		 ************************************************************/
		public function ajax_gridsearch() {

        	check_ajax_referer( 'cfsgrid_search_nonce', 'nonce' );
			
			// ajax posted params
			$params = array(
				'ajax' => 'gridsearch',
			);			
			// collect configuration
			foreach ($this->POST as $k => $v) {
				if (substr($k, 0, 7) === 'config_') {
					$params[substr($k, 7)] = $v;
				}
			}

			if (!empty($this->POST['text'])) $params['text'] = $this->POST['text'];
			if (!empty($this->POST['cat'])) $params['cat'] = $this->POST['cat'];
			if (!empty($this->POST['tag'])) $params['tag'] = $this->POST['tag'];
			if (!empty($this->POST['sort'])) $params['sort'] = $this->POST['sort']; 	
			if (!empty($this->POST['mode'])) $params['mode'] = $this->POST['mode']; 	

			$params = apply_filters('cfswp_grid_search_parameters', $params);

			$response = $this->render_grid($params);
			echo $response;
			wp_die();
		}


		/************************************************************
		 * display post grid
		 ************************************************************/
		public function render_grid($attr) {

			$args = shortcode_atts( array(    
					'posttype' => 'post',  
					'ajax' => '',
					'id' => '',
					'class' => '',
					'mode' => '',
					'not_in' => '',
					'exclude_self' => '',
					'cat' => '',
					'cat_and' => '',
					'cat_in' => '',
					'tag' => '',
					'tag_in' => '',
					'text' => '',
					'sort' => 'post_name', 
					'order' => 'ASC',
					'limit' => '-1', 
					'template' => '', 
					'hideempty' => '',
					'itemclass' => '',
					'oddeven' => '',
					'carousel' => '', 
					'group' => '',
					'grouptemplate' => '',
					'searchbar' => '', 
					'searchtemplate' => '',
					'hide_textsearch' => '',
					'show_searchlabels' => '',
					'show_toggleheaders' => '',
					'show_catsearch' => '',
					'show_tagsearch' => '',
					'show_searchall' => '',
					'show_searchnumbers' => '',
					'show_sortlinks' => '',
					'show_modelinks' => '',
					'show_navilinks' => '',
					'noresults' => '',
					'nocontent' => '',
				), $attr );

			// grid identifier
			$gridID = $args['gridID'] = $args['posttype'].(!empty($args['cat']) ? '-'.$args['cat'] : '');

			if (empty($args['id'])) {
				$args['id'] = 'grid-'.$gridID;
			}
			if (empty($args['class'])) {
				$args['class'] = 'grid-'.$gridID;
			}

			// presets from URL
			if ($args['ajax'] !== 'gridsearch') {
				if (!empty($this->GET['searchtext'])) {
					$args['preset']['text'] = $this->GET['searchtext'];
				}		
				if (!empty($this->GET['searchcat'])) {
					$args['preset']['cat'] = $this->GET['searchcat'];
				}
				if (!empty($this->GET['searchtag'])) {
					$args['preset']['tag'] = $this->GET['searchtag'] ;
				}
				if (!empty($this->GET['searchmode'])) {
					$args['preset']['mode'] = $this->GET['searchmode'] ;
				}
			}
			
			$args = apply_filters('cfswp_grid_attributes', $args);

			// get entries by params
			$entries = array();
			$params = $this->get_grid_parameters($args);	

			if ($args['posttype'] == 'post_categories') {
				$entries = get_categories( $params );
			}
			else {
				$entries = get_posts( $params );
			}

			if (count($entries)==0 && !empty($args['hideempty'])) {
				return '';
			}

			ob_start();

			// HANDLE SEARCHBAR
			$searchlabels = $this->render_grid_searchbar($args, $params, $entries);	

			// RENDER CONTAINER DIV
			if ($args['ajax'] !== 'gridsearch') {
				$class = 'cfs-grid '.(!empty($args['carousel']) ? 'cfs-carousel-grid ' : '').$args['class'];
				echo '<div id="'.$args['id'].'" class="'.$class.'">';	
			}

			// RENDER SEARCH LABELS
			if (!empty($args['searchbar']) && !empty($args['show_searchlabels']) && !empty($searchlabels)) {
				echo '<div class="searchlabels">';
				foreach ($searchlabels as $tgt=>$name) {
					echo '<div class="searchlabel">';
					echo $name.'<a href="javascript:cfswp_resetSearchLabel(\''.$args['id'].'\', \''.$tgt.'\');" title="reset"><i class="far fa-times"></i></a>';
					echo '</div>';
				}
				echo '</div>';	
			}

			// ++++++++++++++++++++++++
			// loop posts
			// ++++++++++++++++++++++++
			if (!empty($entries)) {
				global $post;		

				$i = 0;
				$lastgroup = $currentgroup = '';
				$itemtemplate = $this->get_grid_template('item', $args, true);

				echo '<div class="gridwrapper'.(!empty($args['carousel']) ? ' cfs-carousel' : '').'">';		

				foreach ( $entries as $post ) {

					setup_postdata( $post ); 
					//dbug($post);

					$args['i'] = $i;	
					$args['n'] = count($entries);	

					// RENDER GROUP HEADER
					$lastgroup = $this->render_grid_group($args, $post, $lastgroup);

					// RENDER ITEM
					if (!empty($args['itemclass']) || !empty($args['oddeven'])) {
						$itemclass = "item".(!empty($args['oddeven']) ? ($i%2 ? ' odd' : ' even') : '').(!empty($args['itemclass']) && $args['itemclass']!=='1' ? ' '.$args['itemclass'] : '');
						echo '<div class="'.$itemclass.'" data-id="'.$post->ID.'">';
					}

					load_template($itemtemplate, false, $args);

					if (!empty($args['itemclass']) || !empty($args['oddeven'])) {
						echo '</div>';
					}

					if (!empty($lastgroup) && $i==count($myposts)-1) {
						echo '</div>';
					}	
					$i++;
				}

				echo '</div>';
			}
			else {
				if (!empty($searchlabels)) {
					$msg = !empty($args['noresults']) ? $args['noresults'] : __('Sorry, there are no results.', 'common-feature-set-wp');
					echo '<p class="no-results">'.$msg.'</p>';
				}
				else {
					$msg = !empty($args['nocontent']) ? $args['nocontent'] : __('Sorry, there is no current content.', 'common-feature-set-wp');
					echo '<p class="no-content">'.$msg.'</p>';
				}
			}

			if ($args['ajax'] !== 'gridsearch') {
				echo '</div>';
			}

			wp_reset_postdata();

			return ob_get_clean();	
		}


		/************************************************************
		 * get grid parameters
		 ************************************************************/
		private function get_grid_parameters($args) {
			
			if (!empty($args['preset'])) {
				$args = array_merge($args, $args['preset']);
			}			
			
			if ($args['posttype'] == 'post_categories') {	

				// categories ++++++++++++
				$params = array( 
					'parent' => 0,
					'hide_empty' => !empty($args['hideempty']) ? true : false,
					'orderby' => !empty($args['sort']) ? $args['sort'] : 'term_id',
					'order'   => !empty($args['order']) ? $args['order'] : 'ASC',
				);	
				// get children of cat-slug
				if (!empty($args['cat'])) {
					$parent = get_category_by_slug($args['cat']); 
					$params['parent'] = $parent->term_id;
				}
				// search by text and description
				if (!empty($args['text'])) {
					$params['name__like'] = $params['description__like'] = $args['text'];
				}
				// exclude cat by id
				if (!empty($args['not_in'])) {		
					$params['exclude'] = explode(",", $args['not_in']);
				}
			}
			else {

				// posts ++++++++++++
				$params = array( 
					'post_type' => $args['posttype'],
					'post_status' => 'publish',
					'posts_per_page' => $args['limit'],
					'orderby' => $args['sort'],
					'order'   => $args['order'],
				);

				// search by text and description
				if (!empty($args['text'])) {
					$params['s'] = $args['text'];	
				}
				// get posts by category
				if (!empty($args['cat'])) {
					$params['category_name'] = $args['cat'];
				}
				// get posts by more categories
				if (!empty($args['cat_and'])) {
					$params['category__and'] = explode(",", $args['cat_and']);
					if (!empty($params['category_name'])) {
						$params['category__and'][] = CFSWP_Helper::getCatIdBySlug($params['category_name']);
						unset($params['category_name']);
					}
				}
				// get posts by category selection
				if (!empty($args['cat_in'])) {
					$params['category__in'] = explode(",", $args['cat_in']);
				}
				// get posts by tags
				if (!empty($args['tag'])) {		
					$params['tax_query'] = array(array( 
						'taxonomy' => ($args['posttype'] == 'us_portfolio' ? $args['posttype'] : 'post').'_tag', 
						'field' => 'slug', 
						'terms' => $args['tag'] 
					));
				}
				// get posts by tag selection
				if (!empty($args['tag_in'])) {
					$params['tag__in'] = $args['tag_in'];
				}
				// exclude posts by id
				if (!empty($args['exclude_self']) && is_singular($args['posttype'])) {	
					if ($id = get_the_ID()) {
						$args['not_in'] = $id;
					}
				}
				if (!empty($args['not_in'])) {		
					$params['post__not_in'] = explode(",", $args['not_in']);
				}
			}

			return apply_filters('cfswp_grid_query_parameters', $params, $args);
		}

		/************************************************************
		 * get grid template
		 ************************************************************/
		private function get_grid_template($type, $args, $return = false) {

			// explicit template from shortcode attr
			$template = false;
			if (($type=='item' || $type=='searchbar') && !empty($args['template'])) {
				$template = CFSWP_THEME_DIR . (substr($args['template'], 0, 11) != '/templates/' ? '/templates/' : '') 
						. $args['template'] . (substr($args['template'], -4) != '.php' ? '.php' : '');
			}	

			$theme_id_tpl = CFSWP_THEME_DIR . '/templates/' . $args['id'] . '-' . $type . '.php';

			$theme_class_tpl = CFSWP_THEME_DIR . '/templates/' . str_replace(' ', '_', $args['class']) . '-' . $type . '.php';

			// fallback template from plugin
			if ($type=='item' && $args['posttype']=='post_categories') {
				$plugin_tpl = CFSWP_PLUGIN_DIR . 'templates/post_categories-item.php';
			}
			else {
				$plugin_tpl = CFSWP_PLUGIN_DIR . 'templates/post-' . $type . '.php';
			}

			// take first existing of these template
			$templates = array($template, $theme_id_tpl, $theme_class_tpl, $plugin_tpl);

			foreach ($templates as $tpl) {		
				if (!empty($tpl) && file_exists($tpl)) {
					//echo "<script>console.log('template for ´".$args['id']."´ type ´".$type."´: ', '".explode('wp-content/', $tpl)[1]."');</script>";
					if ($return) {
						return $tpl;
					}			
					load_template($tpl, false, $args);
					break;
				}
			}
		}

		
		/************************************************************
		 * render grid searchbar
		 ************************************************************/
		private function render_grid_searchbar($args, $params, $entries) {

			$searchlabels = array();

			if (!empty($args['searchbar'])) {
				
				// collect search args for search template
				$searchargs = array(
					'id' => $args['id'],
					'class' => $args['class'],
					'cat' => $args['cat'],
					'sort' => $args['sort'],
					'mode' => $args['mode'],
					'template' => $args['searchtemplate'],
					'hide_textsearch' => $args['hide_textsearch'],
					'show_toggleheaders' => $args['show_toggleheaders'],
					'show_catsearch' => $args['show_catsearch'],
					'show_tagsearch' => $args['show_tagsearch'],
					'show_searchall' => $args['show_searchall'],
					'show_searchnumbers' => $args['show_searchnumbers'],
					'show_sortlinks' => $args['show_sortlinks'],
					'show_modelinks' => $args['show_modelinks'],
				);
				
				// collect search labels and presets for display
				if ($args['posttype'] == 'post_categories') {
					// textsearch
					if ($args['posttype'] == 'post_categories' && !empty($params['name__like'])) {			
						$searchlabels['text'] = $searchargs['text'] = $params['name__like'];
					}
					// catsearch
					if (!empty($params['parent'])) {
						$searchargs['category'] = $args['cat'];
						$parent = get_category($params['parent']); 
						$searchargs['cat'] = $parent->name;
						if (!empty($this->POST['cat']) || !empty($this->GET['searchcat'])) {
							$searchlabels['cat'] = $parent->name;
						}
					}
				}
				else {				
					// textsearch	
					if (!empty($params['s'])) {
						$searchlabels['text'] = $searchargs['text'] = $params['s'];
					}	
					// catsearch
					if (!empty($params['category_name'])) {
						$searchargs['category'] = $args['cat'];
						$searchargs['cat'] = $params['category_name'];
						if (!empty($this->POST['cat']) || !empty($this->GET['searchcat'])) {
							$tmp = get_category_by_slug($params['category_name']); 
							$searchlabels['cat'] = $tmp->name;
						}
					}
				}
				// tagsearch
				if (!empty($params['tax_query'])) {
					$searchargs['tag'] = $params['tax_query'][0]['terms'];
					if (!empty($this->POST['tag'])) {
						$tmp = get_term_by($params['tax_query'][0]['field'], $params['tax_query'][0]['terms'], $params['tax_query'][0]['taxonomy']); 
						$searchlabels['tag'] = $tmp->name;
					}
				}

				// filter for e.g. modesearch
				$searchargs = apply_filters('cfswp_grid_search_template_args', $searchargs, $args);
			}
			
			// RENDER SEARCHBAR OR CAROUSEL NAVI
			if ($args['ajax'] !== 'gridsearch') {
				if (!empty($args['searchbar']) || (!empty($args['carousel']) && !empty($args['show_navilinks']))) {
					
					$class = 'cfs-searchbar ' . (!empty($args['carousel']) ? 'cfs-carousel-searchbar ' : '') . $args['class'] . '-searchbar';
					echo '<div id="' . $args['id'] . '-searchbar" class="' . $class . '" data-target="' . $args['id'] . '">';

					// RENDER SEARCHBAR
					if (!empty($args['searchbar'])) {
						foreach ($args as $k=>$v) {
							if ($v !== '' && !is_array($v) && !in_array($k, array('ajax', 'gridID', 'preset', 'n', 'i'))) {
								echo '<input type="hidden" data-target="config" name="' . $k . '" value="' . $v . '" />';
							}
						}
						$this->get_grid_template('searchbar', $searchargs);
					}
					// RENDER CAROUSEL NAVI 
					if (!empty($args['carousel']) && !empty($args['show_navilinks']) && !empty($entries)) {
						
						echo '<div class="navilinks">';
						foreach($entries as $k => $item) {
							echo '<a data-target="navi" data-value="' . $item->post_name . '"'. ($k==0 ? ' class="active"' : '').'>';
							echo $item->post_title;
							echo '</a>';
						}
						echo '</div>';						
					}
						
					echo '</div>';
				}
			}

			return $searchlabels;
		}

		/************************************************************
		 * render grid group headers
		 ************************************************************/
		private function render_grid_group($args, $post, $lastgroup) {

			if (!empty($args['group']) && empty($args['carousel'])) {

				$group = explode(' ', $args['group']);
				if (!empty($group) && !empty($group[0])) {
					$groupvalue = '';
					if (property_exists($post, $group[0])) {
						if (!empty($post[$group[0]])) {
							$groupvalue = $post[$group[0]];
						}
					} 
					else {
						$fields = get_fields( $post->ID );
						if (!empty($fields[$group[0]])) {
							$groupvalue = $fields[$group[0]];
						}
					}
					if (!empty($groupvalue)) {
						if (!empty($group[1])) {
							if ($group[1] == 'date') {
								$pattern = !empty($group[2]) ? $group[2] : 'Y-m-d';
								$currentgroup = date($pattern, strtotime($groupvalue));
							}
							else {
								$currentgroup = $groupvalue;
							}
						}
						else {
							$currentgroup = $groupvalue;
						}
						$currentgroup = str_replace(' ', '', $currentgroup);
						if ($currentgroup != $lastgroup) {
							if ($i!=0) {
								// close latest groupwrapper
								echo '</div>';
							}

							$active = $i==0 ? true : false;
							//$active = $currentgroup >= date('Y-m-d') ? true : false;

							$groupargs = array(
								'id' => $args['id'], 
								'class' => $args['class'], 
								'template' => $args['grouptemplate'], 
								'currentgroup' => $currentgroup, 
								'target' => 'groupwrapper-' . $args['id'] . '-' . $currentgroup, 
								'active' => $active
							);							

							$groupargs = apply_filters('cfswp_grid_group_template_args', $groupargs, $args);

							echo '<div id="groupheader-' . $args['id'] . '-' . $currentgroup . '" class="groupheader">';
							$this->cfswp_grid_template('group', $groupargs);
							echo '</div>';

							echo '<div id="groupwrapper-' . $args['id'] . '-' . $currentgroup . '" class="groupwrapper'.($groupargs['active'] ? ' active' : '').'">';
							$lastgroup = $currentgroup;
						}
					}
				}
			}

			return $lastgroup;
		}


		/************************************************************
		 * GALLERY
		 * - use shortcode in grid item or single templates
		 ************************************************************/
		
		public function render_gallery($attr) {

			$args = shortcode_atts( array(    
					'post_id' => '',  
					'field' => '',
					'include_thumb' => '',
					'first_only' => '',
					'thumbsize' => 'large',
					'bgimg' => '',
					'caption' => '',
					'itemclass' => '',
					'id' => '',
					'class' => '',
					'lightbox' => '',
					'carousel' => '',
					'autowidth' => '',
				), $attr );
			
			$images = $this->get_gallery_images($args['post_id'], $args['include_thumb'], $args['first_only'], $args['thumbsize'], $args['field']);
			
			$res = '';
			if (!empty($images)) {
				$id = !empty($args['id']) ? ' id="'.$args['id'].'"' : '';
				$class = 'cfs-gallery'.(!empty($args['class']) ? ' '.$args['class'] : '')
						.(!empty($args['lightbox']) ? ' cfs-lightbox'.($args['lightbox']=='gallery' ? '-gallery' : '') : '')
						.(!empty($args['carousel']) ? ' cfs-carousel' : '');
				
				$res .= '<div class="' . $class . '"' . $id . '>';
				$res .= $this->display_gallery_images($images, $args['bgimg'], $args['caption'], $args['autowidth'], $args['itemclass']);
				$res .= '</div>';
			}
			
			return $res;
		}
						
		private function get_gallery_images($post_id, $getThumb = false, $getFirst = false, $thumbsize = 'large', $galleryfield = '') {

			$images = array();
			
			// get post id
			if (empty($post_id)) {
				$post_id = get_the_ID();
			}
			
			// get gallery field
			if (empty($galleryfield)) {				
				$fields = get_field_objects($post_id, false, false); 
				foreach ($fields as $fld) {
					if ($fld['type'] == 'gallery') {
						$galleryfield = $fld['name'];
						break;
					}
				}
			}
			
			// get thumbnail
			if (!empty($getThumb)) {
				$thumb = get_the_post_thumbnail_url( $post_id, $thumbsize );
				if (!empty($thumb)) {
					$images[0]['type'] = 'image';
					$images[0]['thumb'] = $thumb;
					$images[0]['full'] = get_the_post_thumbnail_url( $post_id, 'full');
				}
			}
			// get gallery
			if (!$getFirst || empty($images)) { 
				$fields = get_fields( $post_id );
				if (!empty($galleryfield) && !empty($fields[$galleryfield])) {
					foreach ($fields[$galleryfield] as $image) {							
						$img = array(
							'id' => $image['id'],
							'type' => $image['type'],
							'thumb' => $image['url'],
							'full' => $image['url'],
							'caption' => $image['caption'],
							'alt' => $image['alt'],
							'description' => $image['description'],
							'width' => $image['width'],
							'height' => $image['height'],
						);
						if ($image['type'] == 'image') {	
							$img['thumb'] = $image['sizes'][$thumbsize];
						}
						
						array_push($images, $img); 
						if ($getFirst) break;
					}
				}
			} 
			return $images;
		}

		private function display_gallery_images($images, $bgimg = false, $caption = false, $autoWidth = false, $itemclass = '') { 
			$res = '';
			foreach ($images as $img) {			
				
				$itemstyle = $style = $caption = '';
				if (!empty($autoWidth)) {
					$height = intval($autoWidth);
					$width = intval($img['width'] * $height / $img['height']);
					$itemstyle = 'width:'.$width.'px;';	
					//$style .= 'height:'.$height.'px;';	
				}
				
				if ($bgimg && $img['type'] == 'image') {
					$style .= ' style="background-image: url(\''.$img['thumb'].'\')"';				
				}
				if (!empty($img['caption'])) {
					$caption = ' title="'.$img['caption'].'"';
				}				
				
				$res .= '<div class="gallery-item '.($bgimg ? 'bgimg ' : '').$itemclass.'" style="'.$itemstyle.'">';				
				$res .= '<a class="gallery-link '.$img['type'].'-link" href="'.$img['full'].'" style="'.$style.'"'.$caption.'>';
				
				if ($img['type'] == 'image' && !$bgimg) {
					$res .= '<img src="'.$img['thumb'].'" alt="'.(!empty($img['caption']) ? $img['caption'] : '').'" />';
				}
				elseif ($img['type'] == 'video') {
					$res .= '<video height="auto" controls><source src="'.$img['thumb'].'" type="video/mp4"></video>';
				}
				$res .= '</a>';
				
				if (!empty($caption) && !empty($img['caption'])) {
					$res .= '<div class="caption sample">';
					if (strtolower(substr($img['caption'], 0, 4))=='http' || strtolower(substr($img['caption'], 0, 1))=='/') {
						$res .= '<a href="'.$img['caption'].'" target="_blank" rel="noopener">Hörprobe <i class="fal fa-arrow-right"></i></a>';
					}
					else {
						$res .= $img['caption'];
					}
					$res .= '</div>';
				}
				$res .= '</div>';
			} 
			return $res;
		}

		/************************************************************
		 * save 1st gallery image as thumbnail
		 ************************************************************/
		public function save_gallery_to_thumbnail($post_id) {
			$options = get_option( 'cfswp_options' );

			if (!empty($options['cfswp_save_gallery_to_thumbnail'])) {
				$fields = get_field_objects($post_id, false, false);
				foreach ($fields as $fld) {
					if ($fld['type'] == 'gallery') {
						$gallery = get_field($fld['name'], $post_id);
						$has_thumb = has_post_thumbnail($post_id);
						if (!empty($gallery) && !$has_thumb) {
							foreach ($gallery as $k => $v) {
								if ($v['type'] == 'image') {
									$image_id = $v['id']; 
									break;
								}
							}
							set_post_thumbnail($post_id, $image_id);
						}
						break;
					}
				}
			}
		}


		


		/************************************************************
		 *  map Visual Composer Element - CFS-Grid 
		 ************************************************************/
		public function vc_map_element_cfsgrid() {

			// collect posttypes
			$posttypes = array( 
				__( 'Posts' ) => 'post',
				__( 'Categories' ) => 'post_categories'  
			);
			$custom_post_types = get_post_types( array( 'public' => true, '_builtin' => false, 'exclude_from_search' => false), 'objects', 'and' ); 
			foreach ( $custom_post_types  as $cpt ) {
				$posttypes[$cpt->label] = $cpt->name;
			}

			vc_map( array(
				'name'        => esc_html__( 'CFS-Grid', 'common-feature-set-wp' ),
				'description' => esc_html__( 'Renders grids of posts and categories.', 'common-feature-set-wp' ),
				'base'        => 'cfs-grid',
				'params'      => array(
					/*** GENERAL ***/
					array(
						'param_name' => 'posttype',
						'type'       => 'dropdown',
						'value'       => $posttypes,
						'std'         => 'post',
						'group'    => esc_html__( 'General Settings', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Post Type', 'common-feature-set-wp' ),
						'description' => esc_html__( 'Choose a post type as the grid´s data source.', 'common-feature-set-wp' ),
						'admin_label' => true,
					),
					array(
						'param_name' => 'class',
						'type'       => 'textfield',
						'group'    => esc_html__( 'General Settings', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'CSS class', 'common-feature-set-wp' ),
						'admin_label' => true,
						'edit_field_class' => 'vc_col-sm-6',
					),
					array(
						'param_name' => 'id',
						'type'       => 'textfield',
						'group'    => esc_html__( 'General Settings', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'CSS id', 'common-feature-set-wp' ),
						'admin_label' => true,
						'edit_field_class' => 'vc_col-sm-6',
					),
					array(
						'param_name' => 'itemclass',
						'type'       => 'textfield',
						'value'       => 'item',
						'group'    => esc_html__( 'General Settings', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Item class', 'common-feature-set-wp' ),
						'edit_field_class' => 'vc_col-sm-6',
					),
					array(
						'param_name' => 'oddeven',
						'type'       => 'checkbox',
						'group'    => esc_html__( 'General Settings', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Odd/even class', 'common-feature-set-wp' ),
						'edit_field_class' => 'vc_col-sm-6',
					),		
					array(
						'param_name' => 'carousel',
						'type'       => 'checkbox',
						'group'    => esc_html__( 'General Settings', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Carousel', 'common-feature-set-wp' ),
						'edit_field_class' => 'vc_col-sm-6',
					),		
					array(
						'param_name' => 'show_navilinks',
						'type'       => 'checkbox',
						'group'    => esc_html__( 'General Settings', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Display carousel navigation', 'common-feature-set-wp' ),
						'dependency' => array('element' => 'carousel', 'not_empty' => true),
						'edit_field_class' => 'vc_col-sm-6',
					),	
					array(
						'param_name' => 'msg_noresults',
						'type'       => 'textfield',
						'group'    => esc_html__( 'General Settings', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Display message if no results found.', 'common-feature-set-wp' ),
					),
					array(
						'param_name' => 'template',
						'type'       => 'textfield',
						'group'    => esc_html__( 'General Settings', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Item template', 'common-feature-set-wp' ),
					),
					/*** SORT, FILTER AND GROUP ***/	
					array(
						'param_name' => 'sort',
						'type'       => 'textfield',
						'value'       => 'post_name',
						'group'    => esc_html__( 'Sort, Filter and Group', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Sort by', 'common-feature-set-wp' ),
						'edit_field_class' => 'vc_col-sm-6',
					),
					array(
						'param_name' => 'order',
						'type'       => 'dropdown',
						'value'       => array(
							esc_html__( 'ascending', 'common-feature-set-wp' ) => 'ASC',
							esc_html__( 'descending', 'common-feature-set-wp' ) => 'DESC',
						),
						'std'         => 'post',
						'group'    => esc_html__( 'Sort, Filter and Group', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Order', 'common-feature-set-wp' ),
						'edit_field_class' => 'vc_col-sm-6',
					),
					array(
						'param_name' => 'limit',
						'type'       => 'textfield',
						'value'       => '-1',
						'group'    => esc_html__( 'Sort, Filter and Group', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Number of items', 'common-feature-set-wp' ),
						'edit_field_class' => 'vc_col-sm-6',
					),
					array(
						'param_name' => 'hideempty',
						'type'       => 'checkbox',
						'group'    => esc_html__( 'Sort, Filter and Group', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Hide empty results', 'common-feature-set-wp' ),
						'edit_field_class' => 'vc_col-sm-6',
					),
					array(
						'param_name' => 'cat',
						'type'       => 'textfield',
						'group'    => esc_html__( 'Sort, Filter and Group', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Filter by category', 'common-feature-set-wp' ),
						'admin_label' => true,
					),			
					array(
						'param_name' => 'tag',
						'type'       => 'textfield',
						'group'    => esc_html__( 'Sort, Filter and Group', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Filter by tag', 'common-feature-set-wp' ),
						'admin_label' => true,
					),		
					array(
						'param_name' => 'mode',
						'type'       => 'textfield',
						'group'    => esc_html__( 'Sort, Filter and Group', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Custom mode parameter', 'common-feature-set-wp' ),
					),	
					array(
						'param_name' => 'group',
						'type'       => 'textfield',
						'group'    => esc_html__( 'Sort, Filter and Group', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Group by', 'common-feature-set-wp' ),
					),		
					array(
						'param_name' => 'grouptemplate',
						'type'       => 'textfield',
						'group'    => esc_html__( 'Sort, Filter and Group', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Group header template', 'common-feature-set-wp' ),
					),				
					/*** SEARCHBAR ***/		
					array(
						'param_name' => 'searchbar',
						'type'       => 'checkbox',
						'group'    => esc_html__( 'Searchbar', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Display search bar', 'common-feature-set-wp' ),
						'edit_field_class' => 'vc_col-sm-6',
					),			
					array(
						'param_name' => 'hide_textsearch',
						'type'       => 'checkbox',
						'group'    => esc_html__( 'Searchbar', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Hide text search field', 'common-feature-set-wp' ),
						'dependency' => array('element' => 'searchbar', 'not_empty' => true),
						'edit_field_class' => 'vc_col-sm-6',
					),		
					array(
						'param_name' => 'show_catsearch',
						'type'       => 'checkbox',
						'group'    => esc_html__( 'Searchbar', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Search by category', 'common-feature-set-wp' ),
						'dependency' => array('element' => 'searchbar', 'not_empty' => true),
						'edit_field_class' => 'vc_col-sm-6',
					),					
					array(
						'param_name' => 'show_tagsearch',
						'type'       => 'checkbox',
						'group'    => esc_html__( 'Searchbar', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Search by tags', 'common-feature-set-wp' ),
						'dependency' => array('element' => 'searchbar', 'not_empty' => true),
						'edit_field_class' => 'vc_col-sm-6',
					),					
					array(
						'param_name' => 'show_sortlinks',
						'type'       => 'checkbox',
						'group'    => esc_html__( 'Searchbar', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Sortation switch', 'common-feature-set-wp' ),
						'dependency' => array('element' => 'searchbar', 'not_empty' => true),
						'edit_field_class' => 'vc_col-sm-6',
					),			
					array(
						'param_name' => 'show_modelinks',
						'type'       => 'checkbox',
						'group'    => esc_html__( 'Searchbar', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Mode switch', 'common-feature-set-wp' ),
						'dependency' => array('element' => 'searchbar', 'not_empty' => true),
						'edit_field_class' => 'vc_col-sm-6',
					),			
					array(
						'param_name' => 'show_searchlabels',
						'type'       => 'checkbox',
						'group'    => esc_html__( 'Searchbar', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Display active search labels', 'common-feature-set-wp' ),
						'dependency' => array('element' => 'searchbar', 'not_empty' => true),
						'edit_field_class' => 'vc_col-sm-6',
					),		
					array(
						'param_name' => 'show_toggleheaders',
						'type'       => 'checkbox',
						'group'    => esc_html__( 'Searchbar', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Display filter headers', 'common-feature-set-wp' ),
						'dependency' => array('element' => 'searchbar', 'not_empty' => true),
						'edit_field_class' => 'vc_col-sm-6',
					),		
					array(
						'param_name' => 'show_searchall',
						'type'       => 'checkbox',
						'group'    => esc_html__( 'Searchbar', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Display links to all', 'common-feature-set-wp' ),
						'dependency' => array('element' => 'searchbar', 'not_empty' => true),
						'edit_field_class' => 'vc_col-sm-6',
					),					
					array(
						'param_name' => 'show_searchnumbers',
						'type'       => 'checkbox',
						'group'    => esc_html__( 'Searchbar', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Display numbers of results', 'common-feature-set-wp' ),
						'dependency' => array('element' => 'searchbar', 'not_empty' => true),
						'edit_field_class' => 'vc_col-sm-6',
					),	
					array(
						'param_name' => 'searchtemplate',
						'type'       => 'textfield',
						'group'    => esc_html__( 'Searchbar', 'common-feature-set-wp' ),
						'heading'    => esc_html__( 'Search bar template', 'common-feature-set-wp' ),
						'dependency' => array('element' => 'searchbar', 'not_empty' => true),
					),		
				),
			) );
		}
		
		/************************************************************
		 * map Visual Composer Element - CFS-TextSearch
		 ************************************************************/
		public function vc_map_element_textsearch() {

			vc_map( array(
				'name'        => esc_html__( 'CFS-TextSearch', 'common-feature-set-wp' ),
				'description' => esc_html__( 'Filter grid or list items by their items\' text contents.', 'common-feature-set-wp' ),
				'base'        => 'cfs-textsearch',
				'params'      => array(
					array(
						'param_name' => 'target',
						'type'       => 'textfield',
						'heading'    => esc_html__( 'Target selector', 'common-feature-set-wp' ),
						'description' => esc_html__( 'CSS selector to identify the searchable items', 'common-feature-set-wp' ),
						'admin_label' => true,
					),
					array(
						'param_name' => 'placeholder',
						'type'       => 'textfield',
						'heading'    => esc_html__( 'Placeholder', 'common-feature-set-wp' ),
					),
					array(
						'param_name' => 'noresults',
						'type'       => 'textfield',
						'heading'    => esc_html__( 'Message if no results found', 'common-feature-set-wp' ),
					),
					array(
						'param_name' => 'icon',
						'type'       => 'checkbox',
						'heading'    => esc_html__( 'Search icon', 'common-feature-set-wp' ),
					),
					array(
						'param_name' => 'class',
						'type'       => 'textfield',
						'heading'    => esc_html__( 'CSS class', 'common-feature-set-wp' ),
						'edit_field_class' => 'vc_col-sm-6',
					),
					array(
						'param_name' => 'id',
						'type'       => 'textfield',
						'heading'    => esc_html__( 'CSS id', 'common-feature-set-wp' ),
						'edit_field_class' => 'vc_col-sm-6',
					),
				),
			));			
		}


	}
	
	
	$CFSWP = new CFSWP_Features();
	
	/********************************************
	 * global function wrapper to render gallery
	 * in grid item or single templates
	 ********************************************/ 
	if ( ! function_exists( 'cfswp_render_gallery' ) ) {
		function cfswp_render_gallery($args) {
			global $CFSWP;
			return $CFSWP->render_gallery($args);
		}
	}

	/********************************************
	 * print_r wrapper for debugging
	 ********************************************/ 
	if ( ! function_exists( 'dbug' ) ) {
		function dbug($array) {
			echo '<pre>'; print_r($array); echo '</pre>';
		}
	}

}