<?php
/* Do not access this file directly */
defined('ABSPATH') or die('');

if ( ! class_exists( 'CFSWP_Options' ) && is_admin()) {
	
	class CFSWP_Options {
		
		var $features = [];
		var $intro = '';
				
		public function __construct() {
						
			// init feature definition, option fields and descriptions
			require_once CFSWP_PLUGIN_DIR . '/includes/CFSWP_Options_Definition.php';
			
			add_action( 'admin_menu', array( $this, 'set_admin_menu') );
			add_action( 'admin_init', array( $this, 'init_settings') );
		}
			
		
		/**
		 *  Add settings menu page 
		 */
		public function set_admin_menu() {
			add_submenu_page(
				'options-general.php',
				'Common Feature Set WP - Settings',
				'Common Feature Set',
				'administrator',
				'cfswp-settings',
				array($this, 'render_options_page') // options page callback 
			);
		}

		/**
		 * Settings menu page HTML
		 */
		public function render_options_page() {
			// check user capabilities
			if ( ! current_user_can( 'manage_options' ) ) {
				return;
			}

?>
<div class="wrap">
	<h1><?php echo esc_html( get_admin_page_title() ); ?></h1>
	<p><?php echo nl2br($this->intro); ?></p>
	<form action="options.php" method="post">
		<?php
			// output security fields for the registered setting 
			settings_fields( CFSWP_PLUGIN_NAME );
			// output setting sections and their fields
			do_settings_sections( CFSWP_PLUGIN_NAME );
			// output save settings button
			submit_button( 'Save Settings' );
		?>
	</form>
</div>
<?php
		}

		/**
		 * options and settings
		 */
		public function init_settings() {
			
			// Register new option settings
			register_setting( CFSWP_PLUGIN_NAME, 'cfswp_options' );
			
			// Register sections in the settings page.
			foreach ($this->features as $name => $spec) {
				
				// Register settings fields
				if (!empty($spec['fields'])) {
										
					$title = !empty($spec['title']) ? __( $spec['title'], 'common-feature-set-wp' ) : $name;
					add_settings_section( 
						'cfswp_section_'.$name, 	
						$title, 
						array($this, 'settings_section_callback'), 
						CFSWP_PLUGIN_NAME 
					);
					
					foreach ($spec['fields'] as $fldname => $params) {	
						$fldtitle = !empty($params['title']) ? __( $params['title'], 'common-feature-set-wp' ) : $fldname;	
						$fldargs = array(
							'label_for' => 'cfswp_'.\esc_html($fldname), 
							'class'     => 'cfs_row',
							'type'      => !empty($params['type']) ? $params['type'] : 'input',
						);
						if ($params['type'] == 'select' && !empty($params['values'])) {
							$fldargs['values'] = $params['values'];
						}
						if (!empty($params['description'])) {
							$fldargs['description'] = \esc_html($params['description']);
						}
						if (!empty($params['attributes'])) {
							$fldargs['attributes'] = $params['attributes'];
						}
						add_settings_field(
							'cfswp_field_'.$fldname, 
							$fldtitle, 
							array($this, 'settings_field_callback'), 
							CFSWP_PLUGIN_NAME,
							'cfswp_section_'.$name, 
							$fldargs
						);
					}
				}
			}
		}	
		

		/**
		 * Developers section callback function.
		 *
		 * @param array $args  The settings array, defining title, id, callback.
		 */
		public function settings_section_callback( $args ) {
			$id = str_replace('cfswp_section_', '', $args['id']); 
			if (!empty($this->features[$id]['description'])) {
				echo '<p>' . $this->features[$id]['description'] . '<p>';
			}
		}
		
		public function settings_field_callback( $args ) {
			$options = get_option( 'cfswp_options' );
			$labelfor = esc_attr( $args['label_for'] );
			$name = 'cfswp_options[' . $labelfor . ']';			
			$type = isset( $args['type'] ) ? $args['type'] : 'input';			
			$value = isset( $options[ $args['label_for'] ] ) ? $options[ $args['label_for'] ] : '';						
			$description = isset( $args['description'] ) ? $args['description'] : '';
			$attributes = isset( $args['attributes'] ) ? $args['attributes'] : '';
			
			if ($type == 'checkbox') {
				$checked = !empty( $value ) ? \checked( $value, '1', false ) : '' ;
			}
			if ($type == 'select' || $type == 'radio') {
				$values = isset( $args['values'] ) ? $args['values'] : array( 'No', 'Yes' );
			}
			
			// render field
			if ($type == 'input') {
				echo '<label for="' . $labelfor . '">';
				echo '<input type="text" id="' . $labelfor . '" name="' . $name . '" value="' . $value . '" '.$attributes.'/>';
				echo !empty($description) ? '<br>'.$description : '';
				echo '</label>';
			}
			elseif ($type == 'checkbox') {
				echo '<label for="' . $labelfor . '">';
				echo '<input type="checkbox" id="' . $labelfor . '" name="' . $name . '" value="1" '.$checked.' '.$attributes.'/>';
				echo !empty($description) ? ' '.$description : '';
				echo '</label>';
			}
			elseif ($type == 'number') {				
				echo '<label for="' . $labelfor . '">';
				echo '<input type="number" id="' . $labelfor . '" name="' . $name . '" value="' . $value . '" min="0" step="1" '.$attributes.'/>';
				echo !empty($description) ? '<br>'.$description : '';
				echo '</label>';
			}
			elseif ($type == 'select') {				
				echo '<label for="' . $labelfor . '">';
				echo '<select id="' . $labelfor . '" name="' . $name . '" '.$attributes.'>';
				echo '<option value=""></option>';
				foreach ($values as $k => $v) {
					$checked = !empty( $value ) ? \checked( $value, $k, false ) : '' ;
					echo '<option value="'.\esc_html($k).'" '.$checked.'>'.\esc_html($v).'</option>';
				}
				echo '</select>';
				echo !empty($description) ? '<br>'.$description : '';
				echo '</label>';
			}
			elseif ($type == 'radio') {				
				echo '<label for="' . $labelfor . '">';
				foreach ($values as $k => $v) {
					$checked = !empty( $value ) ? \checked( $value, $k, false ) : '' ;
					echo '<span class="radio">';
					echo '<input type="radio" id="' . $labelfor . '" name="' . $name . '" value="' . $k . '" '.$checked.' '.$attributes.'/>';
					echo '&nbsp;'.$v;
					echo '</span>';
				}
				echo !empty($description) ? '<br>'.$description : '';
				echo '</label>';
			}
			elseif ($type == 'text') {		
				echo '<label for="' . $labelfor . '">';
				echo '<textarea id="' . $labelfor . '" name="' . $name . '" '.$attributes.'>' . $value . '</textarea>';
				echo !empty($description) ? '<br>'.$description : '';
				echo '</label>';
			}
		}

	}
	
	
	new CFSWP_Options;
}